@@ -1,4 +1,5 @@
 import type Printer from "../printer";
+import type { PrintJoinOptions } from "../printer";
 import {
   isFor,
   isForStatement,
@@ -102,6 +103,7 @@ function ForXStatement(this: Printer, node: t.ForXStatement) {
     this.word("await");
     this.space();
   }
+  this.printInnerComments(node, false);
   this.token("(");
   this.print(node.left, node);
   this.space();
@@ -259,7 +261,13 @@ export function VariableDeclaration(
     this.space();
   }
 
-  this.word(node.kind);
+  const { kind } = node;
+  this.word(kind);
+  const { _noLineTerminator } = this;
+  if (kind === "using") {
+    // ensure no line break after `using`
+    this._noLineTerminator = true;
+  }
   this.space();
 
   let hasInits = false;
@@ -285,6 +293,16 @@ export function VariableDeclaration(
   //       bar = "foo";
   //
 
+  let iterator: PrintJoinOptions["iterator"] | undefined;
+  if (kind === "using") {
+    // Ensure no line break between `using` and the first declarator
+    iterator = (_, i: number) => {
+      if (i === 0) {
+        this._noLineTerminator = _noLineTerminator;
+      }
+    };
+  }
+
   this.printList(node.declarations, node, {
     separator: hasInits
       ? function (this: Printer) {
@@ -292,6 +310,7 @@ export function VariableDeclaration(
           this.newline();
         }
       : undefined,
+    iterator,
     indent: node.declarations.length > 1 ? true : false,
   });
 
