@@ -79,14 +79,12 @@ interface PrintSequenceOptions extends Partial<AddNewlinesOptions> {
 
 interface PrintListOptions {
   separator?: (this: Printer) => void;
+  iterator?: (node: t.Node, index: number) => void;
   statement?: boolean;
   indent?: boolean;
 }
 
-type PrintJoinOptions = PrintListOptions &
-  PrintSequenceOptions & {
-    iterator?: (node: t.Node, index: number) => void;
-  };
+export type PrintJoinOptions = PrintListOptions & PrintSequenceOptions;
 class Printer {
   constructor(format: Format, map: SourceMap) {
     this.format = format;
@@ -579,6 +577,7 @@ class Printer {
     // trailingCommentsLineOffset also used to check if called from printJoin
     // it will be ignored if `noLineTerminator||this._noLineTerminator`
     trailingCommentsLineOffset?: number,
+    forceParens?: boolean,
   ) {
     if (!node) return;
 
@@ -618,7 +617,9 @@ class Printer {
     this._maybeAddAuxComment(this._insideAux && !oldInAux);
 
     let shouldPrintParens = false;
-    if (
+    if (forceParens) {
+      shouldPrintParens = true;
+    } else if (
       format.retainFunctionParens &&
       nodeType === "FunctionExpression" &&
       node.extra &&
@@ -1070,18 +1071,14 @@ class Printer {
       }
     }
   }
-  // todo(flow->ts): was Node
   printAssertions(node: Extract<t.Node, { assertions?: t.ImportAttribute[] }>) {
-    if (node.assertions?.length) {
-      this.space();
-      this.word("assert");
-      this.space();
-      this.token("{");
-      this.space();
-      this.printList(node.assertions, node);
-      this.space();
-      this.token("}");
-    }
+    this.space();
+    this.printInnerComments(node);
+    this.token("{");
+    this.space();
+    this.printList(node.assertions, node);
+    this.space();
+    this.token("}");
   }
 }
 
