@@ -281,7 +281,7 @@ export default abstract class StatementParser extends ExpressionParser {
     if (!this.isContextual(tt._let)) {
       return false;
     }
-    return this.isLetKeyword(context);
+    return this.hasFollowingIdentifier(context);
   }
 
   /**
@@ -293,7 +293,7 @@ export default abstract class StatementParser extends ExpressionParser {
    * @returns {boolean}
    * @memberof StatementParser
    */
-  isLetKeyword(context?: string | null): boolean {
+  hasFollowingIdentifier(context?: string | null): boolean {
     const next = this.nextTokenStart();
     const nextCh = this.codePointAtPos(next);
     // For ambiguous cases, determine if a LexicalDeclaration (or only a
@@ -326,6 +326,17 @@ export default abstract class StatementParser extends ExpressionParser {
     return false;
   }
 
+  startsUsingForOf(): boolean {
+    const lookahead = this.lookahead();
+    if (lookahead.type === tt._of && !lookahead.containsEsc) {
+      // `using of` must start a for-lhs-of statement
+      return false;
+    } else {
+      this.expectPlugin("explicitResourceManagement");
+      return true;
+    }
+  }
+
   // Parse a single statement.
   //
   // If expecting a statement and finding a slash operator, parse a
@@ -340,25 +351,22 @@ export default abstract class StatementParser extends ExpressionParser {
     context?: string | null,
     topLevel?: boolean,
   ): N.Statement {
+    let decorators: N.Decorator[] | null = null;
+
     if (this.match(tt.at)) {
-      this.parseDecorators(true);
+      decorators = this.parseDecorators(true);
     }
-    return this.parseStatementContent(context, topLevel);
+    return this.parseStatementContent(context, topLevel, decorators);
   }
 
   parseStatementContent(
     this: Parser,
     context?: string | null,
     topLevel?: boolean | null,
+    decorators?: N.Decorator[] | null,
   ): N.Statement {
-    let starttype = this.state.type;
+    const starttype = this.state.type;
     const node = this.startNode();
-    let kind;
-
-    if (this.isLet(context)) {
-      starttype = tt._var;
-      kind = "let";
-    }
 
     // Most types of statements are recognized by the keyword they
     // start with. Many are trivial to parse, some require a bit of
@@ -392,7 +400,13 @@ export default abstract class StatementParser extends ExpressionParser {
 
       case tt._class:
         if (context) this.unexpected();
-        return this.parseClass(node as Undone<N.ClassDeclaration>, true);
+        return this.parseClass(
+          this.maybeTakeDecorators(
+            decorators,
+            node as Undone<N.ClassDeclaration>,
+          ),
+          true,
+        );
 
       case tt._if:
         return this.parseIfStatement(node as Undone<N.IfStatement>);
@@ -405,9 +419,28 @@ export default abstract class StatementParser extends ExpressionParser {
       case tt._try:
         return this.parseTryStatement(node as Undone<N.TryStatement>);
 
+      case tt._using:
+        // using [no LineTerminator here] BindingList[+Using]
+        if (this.hasFollowingLineBreak()) {
+          break;
+        }
+      // fall through
+      case tt._let:
+        if (this.state.containsEsc || !this.hasFollowingIdentifier(context)) {
+          break;
+        }
+      // fall through
       case tt._const:
-      case tt._var:
-        kind = kind || this.state.value;
+      case tt._var: {
+        const kind = this.state.value;
+        if (kind === "using") {
+          this.expectPlugin("explicitResourceManagement");
+          if (!this.scope.inModule && this.scope.inTopLevel) {
+            this.raise(Errors.UnexpectedUsingDeclaration, {
+              at: this.state.startLoc,
+            });
+          }
+        }
         if (context && kind !== "var") {
           this.raise(Errors.UnexpectedLexicalDeclaration, {
             at: this.state.startLoc,
@@ -417,7 +450,7 @@ export default abstract class StatementParser extends ExpressionParser {
           node as Undone<N.VariableDeclaration>,
           kind,
         );
-
+      }
       case tt._while:
         return this.parseWhileStatement(node as Undone<N.WhileStatement>);
       case tt._with:
@@ -462,6 +495,7 @@ export default abstract class StatementParser extends ExpressionParser {
               | N.ExportDefaultDeclaration
               | N.ExportDefaultDeclaration
             >,
+            decorators,
           );
 
           if (
@@ -521,6 +555,7 @@ export default abstract class StatementParser extends ExpressionParser {
       return this.parseExpressionStatement(
         node as Undone<N.ExpressionStatement>,
         expr,
+        decorators,
       );
     }
   }
@@ -531,37 +566,49 @@ export default abstract class StatementParser extends ExpressionParser {
     }
   }
 
-  takeDecorators(node: N.HasDecorators): void {
-    const decorators =
-      this.state.decoratorStack[this.state.decoratorStack.length - 1];
-    if (decorators.length) {
-      node.decorators = decorators;
-      this.resetStartLocationFromNode(node, decorators[0]);
-      this.state.decoratorStack[this.state.decoratorStack.length - 1] = [];
+  decoratorsEnabledBeforeExport(): boolean {
+    if (this.hasPlugin("decorators-legacy")) return true;
+    return (
+      this.hasPlugin("decorators") &&
+      !!this.getPluginOption("decorators", "decoratorsBeforeExport")
+    );
+  }
+
+  // Attach the decorators to the given class.
+  // NOTE: This method changes the .start location of the class, and thus
+  // can affect comment attachment. Calling it before or after finalizing
+  // the class node (and thus finalizing its comments) changes how comments
+  // before the `class` keyword or before the final .start location of the
+  // class are attached.
+  maybeTakeDecorators<T extends Undone<N.Class>>(
+    maybeDecorators: N.Decorator[] | null,
+    classNode: T,
+    exportNode?: Undone<N.ExportDefaultDeclaration | N.ExportNamedDeclaration>,
+  ): T {
+    if (maybeDecorators) {
+      classNode.decorators = maybeDecorators;
+      this.resetStartLocationFromNode(classNode, maybeDecorators[0]);
+      if (exportNode) this.resetStartLocationFromNode(exportNode, classNode);
     }
+    return classNode;
   }
 
   canHaveLeadingDecorator(): boolean {
     return this.match(tt._class);
   }
 
-  parseDecorators(this: Parser, allowExport?: boolean): void {
-    const currentContextDecorators =
-      this.state.decoratorStack[this.state.decoratorStack.length - 1];
-    while (this.match(tt.at)) {
-      const decorator = this.parseDecorator();
-      currentContextDecorators.push(decorator);
-    }
+  parseDecorators(this: Parser, allowExport?: boolean): N.Decorator[] {
+    const decorators = [];
+    do {
+      decorators.push(this.parseDecorator());
+    } while (this.match(tt.at));
 
     if (this.match(tt._export)) {
       if (!allowExport) {
         this.unexpected();
       }
 
-      if (
-        this.hasPlugin("decorators") &&
-        !this.getPluginOption("decorators", "decoratorsBeforeExport")
-      ) {
+      if (!this.decoratorsEnabledBeforeExport()) {
         this.raise(Errors.DecoratorExportClass, { at: this.state.startLoc });
       }
     } else if (!this.canHaveLeadingDecorator()) {
@@ -569,6 +616,8 @@ export default abstract class StatementParser extends ExpressionParser {
         at: this.state.startLoc,
       });
     }
+
+    return decorators;
   }
 
   parseDecorator(this: Parser): N.Decorator {
@@ -578,10 +627,6 @@ export default abstract class StatementParser extends ExpressionParser {
     this.next();
 
     if (this.hasPlugin("decorators")) {
-      // Every time a decorator class expression is evaluated, a new empty array is pushed onto the stack
-      // So that the decorators of any nested class expressions will be dealt with separately
-      this.state.decoratorStack.push([]);
-
       const startLoc = this.state.startLoc;
       let expr: N.Expression;
 
@@ -624,8 +669,6 @@ export default abstract class StatementParser extends ExpressionParser {
 
         node.expression = this.parseMaybeDecoratorArguments(expr);
       }
-
-      this.state.decoratorStack.pop();
     } else {
       node.expression = this.parseExprSubscripts();
     }
@@ -762,16 +805,26 @@ export default abstract class StatementParser extends ExpressionParser {
     }
 
     const startsWithLet = this.isContextual(tt._let);
-    const isLet = startsWithLet && this.isLetKeyword();
-    if (this.match(tt._var) || this.match(tt._const) || isLet) {
+    const startsWithUsing =
+      this.isContextual(tt._using) && !this.hasFollowingLineBreak();
+    const isLetOrUsing =
+      (startsWithLet && this.hasFollowingIdentifier()) ||
+      (startsWithUsing &&
+        this.hasFollowingIdentifier() &&
+        this.startsUsingForOf());
+    if (this.match(tt._var) || this.match(tt._const) || isLetOrUsing) {
       const initNode = this.startNode<N.VariableDeclaration>();
-      const kind = isLet ? "let" : this.state.value;
+      const kind = this.state.value;
       this.next();
       this.parseVar(initNode, true, kind);
       const init = this.finishNode(initNode, "VariableDeclaration");
 
+      const isForIn = this.match(tt._in);
+      if (isForIn && startsWithUsing) {
+        this.raise(Errors.ForInUsing, { at: init });
+      }
       if (
-        (this.match(tt._in) || this.isContextual(tt._of)) &&
+        (isForIn || this.isContextual(tt._of)) &&
         init.declarations.length === 1
       ) {
         return this.parseForIn(node as Undone<N.ForInOf>, init, awaitAt);
@@ -989,7 +1042,7 @@ export default abstract class StatementParser extends ExpressionParser {
   parseVarStatement(
     this: Parser,
     node: Undone<N.VariableDeclaration>,
-    kind: "var" | "let" | "const",
+    kind: "var" | "let" | "const" | "using",
     allowMissingInitializer: boolean = false,
   ): N.VariableDeclaration {
     this.next();
@@ -1102,6 +1155,8 @@ export default abstract class StatementParser extends ExpressionParser {
   parseExpressionStatement(
     node: Undone<N.ExpressionStatement>,
     expr: N.Expression,
+    /* eslint-disable-next-line @typescript-eslint/no-unused-vars -- used in TypeScript parser */
+    decorators: N.Decorator[] | null,
   ) {
     node.expression = expr;
     this.semicolon();
@@ -1317,7 +1372,7 @@ export default abstract class StatementParser extends ExpressionParser {
     this: Parser,
     node: Undone<N.VariableDeclaration>,
     isFor: boolean,
-    kind: "var" | "let" | "const",
+    kind: "var" | "let" | "const" | "using",
     allowMissingInitializer: boolean = false,
   ): Undone<N.VariableDeclaration> {
     const declarations: N.VariableDeclarator[] = (node.declarations = []);
@@ -1359,13 +1414,17 @@ export default abstract class StatementParser extends ExpressionParser {
   parseVarId(
     this: Parser,
     decl: Undone<N.VariableDeclarator>,
-    kind: "var" | "let" | "const",
+    kind: "var" | "let" | "const" | "using",
   ): void {
-    decl.id = this.parseBindingAtom();
-    this.checkLVal(decl.id, {
+    const id = this.parseBindingAtom();
+    if (kind === "using" && id.type !== "Identifier") {
+      this.raise(Errors.UsingDeclarationHasBindingPattern, { at: id });
+    }
+    this.checkLVal(id, {
       in: { type: "VariableDeclarator" },
       binding: kind === "var" ? BIND_VAR : BIND_LEXICAL,
     });
+    decl.id = id;
   }
 
   // Parse a function declaration or literal (depending on the
@@ -1480,8 +1539,7 @@ export default abstract class StatementParser extends ExpressionParser {
     isStatement: /* T === ClassDeclaration */ boolean,
     optionalId?: boolean,
   ): T {
-    this.next();
-    this.takeDecorators(node);
+    this.next(); // 'class'
 
     // A class definition is always strict mode code.
     const oldStrict = this.state.strict;
@@ -2110,6 +2168,7 @@ export default abstract class StatementParser extends ExpressionParser {
       | N.ExportAllDeclaration
       | N.ExportNamedDeclaration
     >,
+    decorators: N.Decorator[] | null,
   ): N.AnyExport {
     const hasDefault = this.maybeParseExportDefaultSpecifier(
       // @ts-expect-error todo(flow->ts)
@@ -2134,6 +2193,9 @@ export default abstract class StatementParser extends ExpressionParser {
 
     if (hasStar && !hasNamespace) {
       if (hasDefault) this.unexpected();
+      if (decorators) {
+        throw this.raise(Errors.UnsupportedDecoratorExport, { at: node });
+      }
       this.parseExportFrom(node as Undone<N.ExportNamedDeclaration>, true);
 
       return this.finishNode(node, "ExportAllDeclaration");
@@ -2154,6 +2216,9 @@ export default abstract class StatementParser extends ExpressionParser {
     let hasDeclaration;
     if (isFromRequired || hasSpecifiers) {
       hasDeclaration = false;
+      if (decorators) {
+        throw this.raise(Errors.UnsupportedDecoratorExport, { at: node });
+      }
       this.parseExportFrom(
         node as Undone<N.ExportNamedDeclaration>,
         isFromRequired,
@@ -2165,22 +2230,31 @@ export default abstract class StatementParser extends ExpressionParser {
     }
 
     if (isFromRequired || hasSpecifiers || hasDeclaration) {
-      this.checkExport(
-        node as Undone<N.ExportNamedDeclaration>,
-        true,
-        false,
-        !!(node as Undone<N.ExportNamedDeclaration>).source,
-      );
-      return this.finishNode(node, "ExportNamedDeclaration");
+      const node2 = node as Undone<N.ExportNamedDeclaration>;
+      this.checkExport(node2, true, false, !!node2.source);
+      if (node2.declaration?.type === "ClassDeclaration") {
+        this.maybeTakeDecorators(decorators, node2.declaration, node2);
+      } else if (decorators) {
+        throw this.raise(Errors.UnsupportedDecoratorExport, { at: node });
+      }
+      return this.finishNode(node2, "ExportNamedDeclaration");
     }
 
     if (this.eat(tt._default)) {
+      const node2 = node as Undone<N.ExportDefaultDeclaration>;
       // export default ...
-      (node as Undone<N.ExportDefaultDeclaration>).declaration =
-        this.parseExportDefaultExpression();
-      this.checkExport(node as Undone<N.ExportDefaultDeclaration>, true, true);
+      const decl = this.parseExportDefaultExpression();
+      node2.declaration = decl;
 
-      return this.finishNode(node, "ExportDefaultDeclaration");
+      if (decl.type === "ClassDeclaration") {
+        this.maybeTakeDecorators(decorators, decl as N.ClassDeclaration, node2);
+      } else if (decorators) {
+        throw this.raise(Errors.UnsupportedDecoratorExport, { at: node });
+      }
+
+      this.checkExport(node2, true, true);
+
+      return this.finishNode(node2, "ExportDefaultDeclaration");
     }
 
     throw this.unexpected(null, tt.braceL);
@@ -2291,8 +2365,14 @@ export default abstract class StatementParser extends ExpressionParser {
       ) {
         this.raise(Errors.DecoratorBeforeExport, { at: this.state.startLoc });
       }
-      this.parseDecorators(false);
-      return this.parseClass(expr as Undone<N.ClassExpression>, true, true);
+      return this.parseClass(
+        this.maybeTakeDecorators(
+          this.parseDecorators(false),
+          this.startNode<N.ClassDeclaration>(),
+        ),
+        true,
+        true,
+      );
     }
 
     if (this.match(tt._const) || this.match(tt._var) || this.isLet()) {
@@ -2311,6 +2391,14 @@ export default abstract class StatementParser extends ExpressionParser {
     // eslint-disable-next-line @typescript-eslint/no-unused-vars
     node: Undone<N.ExportNamedDeclaration>,
   ): N.Declaration | undefined | null {
+    if (this.match(tt._class)) {
+      const node = this.parseClass(
+        this.startNode<N.ClassDeclaration>(),
+        true,
+        false,
+      );
+      return node;
+    }
     return this.parseStatement(null) as N.Declaration;
   }
 
@@ -2474,14 +2562,6 @@ export default abstract class StatementParser extends ExpressionParser {
         }
       }
     }
-
-    const currentContextDecorators =
-      this.state.decoratorStack[this.state.decoratorStack.length - 1];
-    // If node.declaration is a class, it will take all decorators in the current context.
-    // Thus we should throw if we see non-empty decorators here.
-    if (currentContextDecorators.length) {
-      throw this.raise(Errors.UnsupportedDecoratorExport, { at: node });
-    }
   }
 
   checkDeclaration(node: N.Pattern | N.ObjectProperty): void {
@@ -2612,6 +2692,24 @@ export default abstract class StatementParser extends ExpressionParser {
     return false;
   }
 
+  checkImportReflection(node: Undone<N.ImportDeclaration>) {
+    if (node.module) {
+      if (
+        node.specifiers.length !== 1 ||
+        node.specifiers[0].type !== "ImportDefaultSpecifier"
+      ) {
+        this.raise(Errors.ImportReflectionNotBinding, {
+          at: node.specifiers[0].loc.start,
+        });
+      }
+      if (node.assertions?.length > 0) {
+        this.raise(Errors.ImportReflectionHasAssertion, {
+          at: node.specifiers[0].loc.start,
+        });
+      }
+    }
+  }
+
   checkJSONModuleImport(
     node: Undone<
       N.ExportAllDeclaration | N.ExportNamedDeclaration | N.ImportDeclaration
@@ -2645,6 +2743,39 @@ export default abstract class StatementParser extends ExpressionParser {
     }
   }
 
+  parseMaybeImportReflection(node: Undone<N.ImportDeclaration>) {
+    let isImportReflection = false;
+    if (this.isContextual(tt._module)) {
+      const lookahead = this.lookahead();
+      if (tokenIsIdentifier(lookahead.type)) {
+        if (lookahead.type !== tt._from) {
+          // import module x
+          isImportReflection = true;
+        } else {
+          const nextNextTokenFirstChar = this.input.charCodeAt(
+            this.nextTokenStartSince(lookahead.end),
+          );
+          if (nextNextTokenFirstChar === charCodes.lowercaseF) {
+            // import module from from ...
+            isImportReflection = true;
+          }
+        }
+      } else {
+        // import module { x } ...
+        // This is invalid, we will continue parsing and throw
+        // a recoverable error later
+        isImportReflection = true;
+      }
+    }
+    if (isImportReflection) {
+      this.expectPlugin("importReflection");
+      this.next(); // eat tt._module;
+      node.module = true;
+    } else if (this.hasPlugin("importReflection")) {
+      node.module = false;
+    }
+  }
+
   // Parses import declaration.
   // https://tc39.es/ecma262/#prod-ImportDeclaration
 
@@ -2652,6 +2783,7 @@ export default abstract class StatementParser extends ExpressionParser {
     // import '...'
     node.specifiers = [];
     if (!this.match(tt.string)) {
+      this.parseMaybeImportReflection(node);
       // check if we have a default import like
       // import React from "react";
       const hasDefault = this.maybeParseDefaultImportSpecifier(node);
@@ -2684,6 +2816,7 @@ export default abstract class StatementParser extends ExpressionParser {
         node.attributes = attributes;
       }
     }
+    this.checkImportReflection(node);
     this.checkJSONModuleImport(node);
 
     this.semicolon();
